@@ -1,14 +1,37 @@
-import sys
-import traceback
+#
+#    CASPy - A program that provides both a GUI and a CLI to SymPy.
+#    Copyright (C) 2020 Folke Ishii
+#
+#    This program is free software: you can redistribute it and/or modify
+#    it under the terms of the GNU General Public License as published by
+#    the Free Software Foundation, either version 3 of the License, or
+#    (at your option) any later version.
+#
+#    This program is distributed in the hope that it will be useful,
+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#    GNU General Public License for more details.
+#
+#    You should have received a copy of the GNU General Public License
+#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
+#
 
-import click
 from PyQt5.QtCore import QObject, QThreadPool
 from PyQt5.QtWidgets import QApplication
 
+from .qt_assets.tabs.worker import BaseWorker
+
+import typing as ty
+import traceback
+import click
+import sys
+
 
 class Cli(QObject):
-    def __init__(self, command, params, copy, worker, parent=None):
-        super(Cli, self).__init__(parent)
+    def __init__(
+        self, command: str, params: list, copy: int, worker: ty.Type[BaseWorker]
+    ) -> None:
+        super(Cli, self).__init__()
 
         self.command = command
         self.params = params
@@ -18,10 +41,10 @@ class Cli(QObject):
 
         self.threadpool = QThreadPool()
 
-    def stop_thread(self):
+    def stop_thread(self) -> None:
         pass
 
-    def call_worker(self):
+    def call_worker(self) -> None:
         """
         Call worker, send command and params, and then start thread
         """
@@ -31,7 +54,7 @@ class Cli(QObject):
 
         self.threadpool.start(worker)
 
-    def print_output(self, input_dict):
+    def print_output(self, input_dict: dict) -> None:
         """
         Prints output then exit program
         :param input_dict: dict
@@ -47,55 +70,137 @@ class Cli(QObject):
         sys.exit()
 
 
+def suppress_qt_warnings() -> None:
+    from os import environ
+
+    environ["QT_DEVICE_PIXEL_RATIO"] = "0"
+    environ["QT_AUTO_SCREEN_SCALE_FACTOR"] = "1"
+    environ["QT_SCREEN_SCALE_FACTORS"] = "1"
+    environ["QT_SCALE_FACTOR"] = "1"
+
+
 class EncloseNegative(click.Command):
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args, **kwargs) -> None:
         super(EncloseNegative, self).__init__(*args, **kwargs)
+        self.params.insert(
+            0,
+            click.core.Option(
+                ("--dont-suppress",),
+                is_flag=True,
+                default=False,
+                help="Set flag to suppress setting envirmental "
+                "varables in order to suppress the error "
+                "message QT_DEVICE_PIXEL_RATIO.",
+            ),
+        )
+
+    def parse_args(self, ctx: click.core.Context, args: list) -> list:
+        """Enclose every negative number in parentheses so click doesn't think it's an option.
+        Everything inside parentheses will concatenate following string until the parentheses matches.
+        For example:
+        args before:
+            ["caspy", "sin(x**2", " + ", "x**(", " 1/3)", "-1", ")", "-1", "x"]
+        args after:
+            ["caspy", "sin(x**2 + x**( 1/3) -1)", "-1", "x"]
+        """
+        if "--dont-suppress" not in args:
+            suppress_qt_warnings()
+
+        # Throw error if parentheses doesn't match
+        if ("".join(args).count(")") - "".join(args).count("(")) != 0:
+            raise Exception("Parentheses not matching") from SyntaxError
+
+        fixed = []
+        i = 0
+        while i < len(args):
+            temp = args[i]
+            temp_c = 0
+            while (temp.count(")") - temp.count("(")) != 0:
+                temp_c += 1
+                temp += args[i + temp_c]
+            i += temp_c + 1
+            fixed.append(temp)
+
+        args = fixed
 
-    def parse_args(self, ctx, args):
-        """Enclose every negative number in parentheses so click doesn't think it's an option"""
         for arg in args:
             if len(arg) > 1:
                 if arg[0] == "-" and arg[1] in "0123456789()":
                     args[args.index(arg)] = f"({arg})"
 
-                if len(arg) == 3: # -oo negative infinity
+                if len(arg) == 3:  # '-oo' negative infinity
                     if arg == "-oo":
                         args[args.index(arg)] = f"({arg})"
 
         return super(EncloseNegative, self).parse_args(ctx, args)
 
 
-# Default flags, these flags are added to a command by using the decorator '@add_options(DEFAULT_FLAGS)'.
+# Default flags, these flags are added
+# to a command by using the decorator '@add_options(DEFAULT_FLAGS)'.
 DEFAULT_FLAGS = [
-    click.option("--preview", "-p", is_flag=True, default=False, help="Previews instead of evaluates"),
-    click.option("--output-type", "-o", default=1, type=click.IntRange(1, 3),
-                 help="Select output type, 1 for pretty; 2 for latex and 3 for normal"),
-    click.option("--use-unicode", "-u", is_flag=True, default=False, help="Use unicode"),
-    click.option("--line-wrap", "-l", is_flag=True, default=False, help="Use line wrap"),
-    click.option("-c", "--copy", type=click.IntRange(1, 3),
-                 help="Copies the answer. 1 for exact_ans, 2 for approx_ans, and 3 for a list of [exact_ans, "
-                      "approx_ans].")
+    click.option(
+        "--preview",
+        "-p",
+        is_flag=True,
+        default=False,
+        help="Previews instead of evaluates",
+    ),
+    click.option(
+        "--output-type",
+        "-o",
+        default=1,
+        type=click.IntRange(1, 3),
+        help="Select output type, 1 for pretty; 2 for latex and 3 for normal",
+    ),
+    click.option(
+        "--use-unicode", "-u", is_flag=True, default=False, help="Use unicode"
+    ),
+    click.option(
+        "--line-wrap", "-l", is_flag=True, default=False, help="Use line wrap"
+    ),
+    click.option(
+        "--copy",
+        "-c",
+        type=click.IntRange(1, 3),
+        help="Copies the answer. 1 for exact_ans, 2 for approx_ans, and 3 for a list of [exact_ans, "
+        "approx_ans].",
+    ),
 ]
 
 # Default argument(s), these argument(s) are added
 # to a command by using the decorator '@add_options(DEFAULT_ARGUMENTS)'.
 DEFAULT_ARGUMENTS = [
-    click.option("--use-scientific", "-s", type=int, default=None,
-                 help="Notate approximate answer with scientific notation, argument is accuracy"),
-    click.option("--accuracy", "-a", type=int, default=10, help="Accuracy of evaluation")
+    click.option(
+        "--use-scientific",
+        "-s",
+        type=int,
+        default=None,
+        help="Notate approximate answer with scientific notation, argument is accuracy",
+    ),
+    click.option(
+        "--accuracy", "-a", type=int, default=10, help="Accuracy of evaluation"
+    ),
 ]
 
-# Options used by equations (This includes formula), added to command by using the decorator '@add_options(EQ_FLAGS)'.
+# Options used by equations (This includes formula),
+# added to command by using the decorator '@add_options(EQ_FLAGS)'.
 EQ_FLAGS = [
-    click.option("--domain", "-d", default="Complexes", help="Give domain to solve for"),
-    click.option("--verify-domain", "-v", is_flag=True, default=False,
-                 help="Filter out any solutions that isn't in domain. Doesn't work with solveset. "
-                      "This flag must be set in order for domain to work if it solves with solve and not"
-                      "solveset. Needed for system of equations")
+    click.option(
+        "--domain", "-d", default="Complexes", help="Give domain to solve for"
+    ),
+    click.option(
+        "--verify-domain",
+        "-v",
+        is_flag=True,
+        default=False,
+        help="Filter out any solutions that isn't in domain. Doesn't work with solveset. "
+        "This flag must be set in order for domain to work if it solves with solve and not"
+        "solveset. Needed for system of equations",
+    ),
 ]
 
 
-def list_merge(default_params, input_params):
+def list_merge(default_params: list, input_params: list) -> list:
     """
     Merges two lists, uses element from input_params if it is not None, else use element from default_params
 
@@ -120,7 +225,9 @@ def list_merge(default_params, input_params):
     return output_list
 
 
-def validate_inputs(input_kwargs, default_params, input_params, name):
+def validate_inputs(
+    input_kwargs: dict, default_params: list, input_params: tuple, name: str
+) -> dict:
     """
     Validates and restricts some of the inputs:
         1. 'output_type' must be integer between 1 and 3 inclusive
@@ -133,18 +240,16 @@ def validate_inputs(input_kwargs, default_params, input_params, name):
         Default parameters
     :param input_params: tuple
         Params typed by user
-    :param name:
+    :param name: str
         Name of the command
     :return:
         Returns either error along with message if validation failed, or True along with 'pass' if validation passed
     """
 
-    if input_kwargs["output_type"] not in range(1, 4):
-        return {"error": "Output type must be integer between 1 and 3 inclusive. 1 for pretty; 2 for latex and 3 for "
-                         "normal."}
-
     if len(input_params) > len(default_params):
-        return {"error": f"'{name}' commad doesn't take more than {len(default_params)} parameters."}
+        return {
+            "error": f"'{name}' commad doesn't take more than {len(default_params)} parameters."
+        }
 
     if len(input_params) == 0:
         return {"error": f"'{name}' command requires at least one parameter."}
@@ -152,7 +257,7 @@ def validate_inputs(input_kwargs, default_params, input_params, name):
     return {True: "pass"}
 
 
-def add_options(options):
+def add_options(options: list):
     """
     Adds flags and/or arguments to command via decorator: @add_options(list_of_flags_or_arguments)
 
@@ -171,34 +276,34 @@ def add_options(options):
 
 
 @click.group()
-def main(**kwargs):
+def main(**kwargs: dict) -> None:
     pass
 
 
 @main.command()
-def start():
+def start() -> None:
     """
     Start the GUI
     """
     from .qt_gui import main
+
     main()
 
 
 @main.command(cls=EncloseNegative)
 @add_options(DEFAULT_FLAGS)
 @add_options(DEFAULT_ARGUMENTS)
-@click.argument("params", nargs=-1)
-def deriv(params, **kwargs):
+@click.argument("params", nargs=-1, metavar="EXPRESSION VARIABLE [ORDER] [AT_POINT]")
+def deriv(params: list, **kwargs: dict) -> None:
     """Derive a function.
 
-    Usage: caspy deriv EXPRESSION VARIABLE [ORDER] [AT_POINT] [FLAGS]
-
     \b
     Example(s):
     >>> caspy deriv x**x x
     >>> caspy deriv sin(1/x) x 3 pi
     """
     from .qt_assets.tabs.derivative import DerivativeWorker
+
     default_params = ["x", "x", "1", None]
 
     validate_input_dict = validate_inputs(kwargs, default_params, params, "deriv")
@@ -211,8 +316,13 @@ def deriv(params, **kwargs):
         options = [kwargs["output_type"], kwargs["use_unicode"], kwargs["line_wrap"]]
     else:
         prefix = "calc_"
-        options = [kwargs["output_type"], kwargs["use_unicode"], kwargs["line_wrap"], kwargs["use_scientific"],
-                   kwargs["accuracy"]]
+        options = [
+            kwargs["output_type"],
+            kwargs["use_unicode"],
+            kwargs["line_wrap"],
+            kwargs["use_scientific"],
+            kwargs["accuracy"],
+        ]
 
     params_to_send = list_merge(default_params, list(params))
 
@@ -223,20 +333,28 @@ def deriv(params, **kwargs):
 @main.command(cls=EncloseNegative)
 @add_options(DEFAULT_FLAGS)
 @add_options(DEFAULT_ARGUMENTS)
-@click.argument("params", nargs=-1)
-@click.option("--approximate-integral", "-A", is_flag=True, default=False,
-              help="Set flag to approximate integral. This overrides the normal calculation")
-def integ(params, **kwargs):
+@click.argument(
+    "params",
+    nargs=-1,
+    metavar="EXPRESSION VARIABLE [LOWER_BOUND UPPER_BOUND] [APPROXIMATE]",
+)
+@click.option(
+    "--approximate-integral",
+    "-A",
+    is_flag=True,
+    default=False,
+    help="Set flag to approximate integral. This overrides the normal calculation",
+)
+def integ(params: list, **kwargs: dict) -> None:
     """Calculate definite and indefinite integrals of expressions.
 
-    Usage: caspy integ EXPRESSION VARIABLE {LOWER_BOUND UPPER_BOUND} [APPROXIMATE] [FLAGS]
-
     \b
     Example(s):
     >>> caspy integ 1/sqrt(1-x**2) x -1 1
     >>> caspy integ x**x x -1 1 -A
     """
     from .qt_assets.tabs.integral import IntegralWorker
+
     default_params = ["x", "x", None, None]
 
     validate_input_dict = validate_inputs(kwargs, default_params, params, "integ")
@@ -249,9 +367,14 @@ def integ(params, **kwargs):
         options = [kwargs["output_type"], kwargs["use_unicode"], kwargs["line_wrap"]]
     else:
         prefix = "calc_"
-        options = [kwargs["approximate_integral"], kwargs["output_type"], kwargs["use_unicode"], kwargs["line_wrap"],
-                   kwargs["use_scientific"],
-                   kwargs["accuracy"]]
+        options = [
+            kwargs["approximate_integral"],
+            kwargs["output_type"],
+            kwargs["use_unicode"],
+            kwargs["line_wrap"],
+            kwargs["use_scientific"],
+            kwargs["accuracy"],
+        ]
 
     params_to_send = list_merge(default_params, list(params))
 
@@ -262,18 +385,17 @@ def integ(params, **kwargs):
 @main.command(cls=EncloseNegative)
 @add_options(DEFAULT_FLAGS)
 @add_options(DEFAULT_ARGUMENTS)
-@click.argument("params", nargs=-1)
-def sum(params, **kwargs):
+@click.argument("params", nargs=-1, metavar="EXPRESSION VARIABLE START END")
+def sum(params: list, **kwargs: dict) -> None:
     """Calculate the summation of an expression.
 
-    Usage: caspy sum EXPRESSION VARIABLE START END [FLAGS]
-
     \b
     Example(s):
     >>> caspy sum x**k/factorial(k) k 0 oo
     >>> caspy sum k**2 k 1 m
     """
     from .qt_assets.tabs.summation import SummationWorker
+
     default_params = ["k", "k", 1, 2]
 
     validate_input_dict = validate_inputs(kwargs, default_params, params, "sum")
@@ -286,9 +408,13 @@ def sum(params, **kwargs):
         options = [kwargs["output_type"], kwargs["use_unicode"], kwargs["line_wrap"]]
     else:
         prefix = "calc_"
-        options = [kwargs["output_type"], kwargs["use_unicode"], kwargs["line_wrap"],
-                   kwargs["use_scientific"],
-                   kwargs["accuracy"]]
+        options = [
+            kwargs["output_type"],
+            kwargs["use_unicode"],
+            kwargs["line_wrap"],
+            kwargs["use_scientific"],
+            kwargs["accuracy"],
+        ]
 
     params_to_send = list_merge(default_params, list(params))
 
@@ -299,18 +425,19 @@ def sum(params, **kwargs):
 @main.command(cls=EncloseNegative)
 @add_options(DEFAULT_FLAGS)
 @add_options(DEFAULT_ARGUMENTS)
-@click.argument("params", nargs=-1)
-def limit(params, **kwargs):
+@click.argument(
+    "params", nargs=-1, metavar="EXPRESSION VARIABLE AS_VARIABLE_IS_APPROACHING [SIDE]"
+)
+def limit(params: list, **kwargs: dict) -> None:
     """Calculate the limit of an expression.
 
-    Usage: caspy limit EXPRESSION VARIABLE AS_VARIABLE_IS_APPROACHING [SIDE] [FLAGS]
-
     \b
     Example(s):
     >>> caspy limit (1+1/(a*n))**(b*n) n oo
     >>> caspy limit n!**(1/n) n 0 -
     """
     from .qt_assets.tabs.limit import LimitWorker
+
     default_params = ["x", "x", 0, "+-"]
 
     validate_input_dict = validate_inputs(kwargs, default_params, params, "limit")
@@ -323,8 +450,13 @@ def limit(params, **kwargs):
         options = [kwargs["output_type"], kwargs["use_unicode"], kwargs["line_wrap"]]
     else:
         prefix = "calc_"
-        options = [kwargs["output_type"], kwargs["use_unicode"], kwargs["line_wrap"], kwargs["use_scientific"],
-                   kwargs["accuracy"]]
+        options = [
+            kwargs["output_type"],
+            kwargs["use_unicode"],
+            kwargs["line_wrap"],
+            kwargs["use_scientific"],
+            kwargs["accuracy"],
+        ]
 
     params_to_send = list_merge(default_params, list(params))
 
@@ -336,23 +468,37 @@ def limit(params, **kwargs):
 @add_options(DEFAULT_FLAGS)
 @add_options(DEFAULT_ARGUMENTS)
 @add_options(EQ_FLAGS)
-@click.argument("params", nargs=-1)
-@click.option("--solve-type", "-st", is_flag=True, default=False,
-              help="Solves an equation with either solve or solveset (see SymPy solve vs solveset). Default is solve, "
-                   "set flag to solve with solveset.")
-def eq(params, **kwargs):
+@click.argument(
+    "params",
+    nargs=-1,
+    metavar="LEFT_EXPRESSION RIGHT_EXPRESSION VARIABLE_TO_SOLVE_FOR [SOLVE_TYPE]",
+)
+@click.option(
+    "--approximate-equation", "-A", type=str,
+    help="Approximate answer to equation. Argument is starting vector"
+)
+@click.option(
+    "--solve-type",
+    "-st",
+    is_flag=True,
+    default=False,
+    help="Solves an equation with either solve or "
+    "solveset (see SymPy solve vs solveset). "
+    "Default is solve, set flag to solve with "
+    "solveset.",
+)
+def eq(params: list, **kwargs: dict) -> None:
     """Solves a normal equation.
 
     Separate equation by either a space or a =, but not both.
 
-    Usage: eq LEFT_EXPRESSION RIGHT_EXPRESSION VARIABLE_TO_SOLVE_FOR [SOLVE_TYPE] [FLAGS]
-
     \b
     Example(s):
     >>> caspy eq x**x 2 x
     >>> caspy eq sin(x)=1 x -st
     """
     from .qt_assets.tabs.equations import EquationsWorker
+
     default_params = ["x", 0, "x"]
 
     if "=" in params[0]:
@@ -368,11 +514,25 @@ def eq(params, **kwargs):
 
     if kwargs["preview"]:
         prefix = "prev_"
-        options = [kwargs["domain"], kwargs["output_type"], kwargs["use_unicode"], kwargs["line_wrap"]]
+        options = [
+            kwargs["domain"],
+            kwargs["output_type"],
+            kwargs["use_unicode"],
+            kwargs["line_wrap"],
+        ]
     else:
         prefix = "calc_"
-        options = [kwargs["solve_type"], kwargs["domain"], kwargs["output_type"], kwargs["use_unicode"],
-                   kwargs["line_wrap"], kwargs["use_scientific"], kwargs["accuracy"], kwargs["verify_domain"]]
+        options = [
+            kwargs["solve_type"],
+            kwargs["domain"],
+            kwargs["output_type"],
+            kwargs["use_unicode"],
+            kwargs["line_wrap"],
+            kwargs["use_scientific"],
+            kwargs["accuracy"],
+            kwargs["verify_domain"],
+            kwargs["approximate_equation"],
+        ]
 
     params_to_send = list_merge(default_params, list(params))
 
@@ -383,21 +543,25 @@ def eq(params, **kwargs):
 @main.command(cls=EncloseNegative)
 @add_options(DEFAULT_FLAGS)
 @add_options(DEFAULT_ARGUMENTS)
-@click.option("--hint", "-h", default="", help="The solving method that you want dsolve to use.")
-@click.argument("params", nargs=-1)
-def diff_eq(params, **kwargs):
+@click.option(
+    "--hint", "-h", default="", help="The solving method that you want dsolve to use."
+)
+@click.argument(
+    "params",
+    nargs=-1,
+    metavar="LEFT_EXPRESSION RIGHT_EXPRESSION FUNCTION_TO_SOLVE_FOR [HINT]",
+)
+def diff_eq(params: list, **kwargs: dict) -> None:
     """Solves a differential equation equation.
-
     Separate equation by either a space or a =, but not both.
 
-    Usage: diff-eq LEFT_EXPRESSION RIGHT_EXPRESSION FUNCTION_TO_SOLVE_FOR [HINT] [FLAGS]
-
     \b
     Example(s):
     >>> caspy diff-eq f'(x) 1/f(x) f(x)
     >>> caspy diff-eq f''(x)+3*f'(x)=x**2 f(x)
     """
     from .qt_assets.tabs.equations import EquationsWorker
+
     default_params = ["f(x)", "f(x)", "f(x)"]
 
     if "=" in params[0]:
@@ -416,8 +580,13 @@ def diff_eq(params, **kwargs):
         options = [kwargs["output_type"], kwargs["use_unicode"], kwargs["line_wrap"]]
     else:
         prefix = "calc_"
-        options = [kwargs["output_type"], kwargs["use_unicode"],
-                   kwargs["line_wrap"], kwargs["use_scientific"], kwargs["accuracy"]]
+        options = [
+            kwargs["output_type"],
+            kwargs["use_unicode"],
+            kwargs["line_wrap"],
+            kwargs["use_scientific"],
+            kwargs["accuracy"],
+        ]
 
     params_to_send = list_merge(default_params, list(params))
 
@@ -433,17 +602,19 @@ def diff_eq(params, **kwargs):
 @add_options(DEFAULT_FLAGS)
 @add_options(DEFAULT_ARGUMENTS)
 @add_options(EQ_FLAGS)
-@click.option("--solve_type", "-st", is_flag=True, default=False,
-              help="Solve either a system of normal equations or a system of differential equations. Defualt if normal,"
-                   " set flag to solve a system of differential equations.")
-@click.argument("no_of_eq", type=int)
-def sys_eq(no_of_eq, **kwargs):
+@click.option(
+    "--solve_type",
+    "-st",
+    is_flag=True,
+    default=False,
+    help="Solve either a system of normal equations or a system of differential equations. Defualt if normal,"
+    " set flag to solve a system of differential equations.",
+)
+@click.argument("no_of_eq", type=int, metavar="sys-eq NO_OF_EQUATIONS [SOLVE_TYPE]")
+def sys_eq(no_of_eq: int, **kwargs: dict) -> None:
     """Solves a system of either normal or differential equations.
-
     Takes number of equations as argument, then will prompt user for all equations
 
-    Usage: sys-eq NO_OF_EQUATIONS [SOLVE_TYPE] [FLAGS]
-
     \b
     Example(s):
     >>> caspy sys-eq 5
@@ -461,17 +632,37 @@ def sys_eq(no_of_eq, **kwargs):
         equation = input(f"Enter equation number {i + 1} of {no_of_eq}: ")
         equations.append(equation)
 
-    variables = input(f"Enter variables to solve for separated by anything other than a-z, 0-9, (), and _: ")
+    variables = input(
+        f"Enter variables to solve for separated by anything other than a-z, 0-9, (), and _: "
+    )
 
     if kwargs["preview"]:
         prefix = "prev_"
-        options = [kwargs["domain"], solve_type, kwargs["output_type"], kwargs["use_unicode"], kwargs["line_wrap"]]
+        options = [
+            kwargs["domain"],
+            solve_type,
+            kwargs["output_type"],
+            kwargs["use_unicode"],
+            kwargs["line_wrap"],
+        ]
     else:
         prefix = "calc_"
-        options = [kwargs["domain"], solve_type, kwargs["output_type"], kwargs["use_unicode"], kwargs["line_wrap"],
-                   kwargs["use_scientific"], kwargs["accuracy"], kwargs["verify_domain"]]
+        options = [
+            kwargs["domain"],
+            solve_type,
+            kwargs["output_type"],
+            kwargs["use_unicode"],
+            kwargs["line_wrap"],
+            kwargs["use_scientific"],
+            kwargs["accuracy"],
+            kwargs["verify_domain"],
+        ]
 
-    to_send = [prefix + "system_eq", [equations] + [variables] + options, kwargs["copy"]]
+    to_send = [
+        prefix + "system_eq",
+        [equations] + [variables] + options,
+        kwargs["copy"],
+    ]
 
     send_to_thread(to_send, EquationsWorker)
 
@@ -479,16 +670,15 @@ def sys_eq(no_of_eq, **kwargs):
 @main.command(cls=EncloseNegative)
 @add_options(DEFAULT_FLAGS)
 @click.argument("expression")
-def simp(expression, **kwargs):
+def simp(expression: str, **kwargs: dict) -> None:
     """Simplifies an expression.
 
-    Usage: simp EXPRESSION [FLAGS]
-
     \b
     Example(s):
     >>> caspy simp sin(x)**2+cos(x)**2
     """
     from .qt_assets.tabs.simplify import SimpWorker
+
     expression = tuple([expression])
 
     default_params = ["x"]
@@ -513,16 +703,15 @@ def simp(expression, **kwargs):
 @main.command(cls=EncloseNegative)
 @add_options(DEFAULT_FLAGS)
 @click.argument("expression")
-def exp(expression, **kwargs):
+def exp(expression: str, **kwargs: dict) -> None:
     """Expandes an expression.
 
-    Usage: exp EXPRESSION [FLAGS]
-
     \b
     Example(s):
     >>> caspy exp (a+b-c)**3
     """
     from .qt_assets.tabs.expand import ExpandWorker
+
     default_params = ["x"]
     expression = tuple([expression])
     validate_input_dict = validate_inputs(kwargs, default_params, expression, "exp")
@@ -547,7 +736,7 @@ def exp(expression, **kwargs):
 @add_options(DEFAULT_ARGUMENTS)
 @click.argument("expression")
 @click.argument("vars_sub", required=False, nargs=-1)
-def eval(expression, vars_sub, **kwargs):
+def eval(expression: str, vars_sub: list, **kwargs: dict) -> None:
     """Evaluates an expression.
 
     After expression you can also subtitute your variables with a value.
@@ -559,14 +748,13 @@ def eval(expression, vars_sub, **kwargs):
     => 3**((3)+(5))
     => 6561
 
-    Usage: eval EXPRESSION [VARS_SUB ... ] [FLAGS]
-
     \b
     Example(s):
     >>> caspy eval exp(pi)+3/sin(6)
     >>> caspy eval 3**x x 3
     """
     from .qt_assets.tabs.evaluate import EvaluateWorker
+
     default_params = ["1+1"]
     expression = tuple([expression])
     validate_input_dict = validate_inputs(kwargs, default_params, expression, "eval")
@@ -575,7 +763,10 @@ def eval(expression, vars_sub, **kwargs):
         return
 
     if len(vars_sub) % 2 != 0:
-        print("Variable substitution must consist of an even number of arguments, see 'eval --help' for more information")
+        print(
+            "Variable substitution must consist of an even number of arguments, see 'eval --help' for more "
+            "information"
+        )
         return
 
     var_sub = ""
@@ -587,56 +778,69 @@ def eval(expression, vars_sub, **kwargs):
         options = [kwargs["output_type"], kwargs["use_unicode"], kwargs["line_wrap"]]
     else:
         prefix = ""
-        options = [kwargs["output_type"], kwargs["use_unicode"], kwargs["line_wrap"], kwargs["use_scientific"],
-                   kwargs["accuracy"]]
+        options = [
+            kwargs["output_type"],
+            kwargs["use_unicode"],
+            kwargs["line_wrap"],
+            kwargs["use_scientific"],
+            kwargs["accuracy"],
+        ]
 
     params_to_send = list_merge(default_params, list(expression))
-    to_send = [prefix + "eval_exp", params_to_send + [var_sub] + options, kwargs["copy"]]
+    to_send = [
+        prefix + "eval_exp",
+        params_to_send + [var_sub] + options,
+        kwargs["copy"],
+    ]
 
     send_to_thread(to_send, EvaluateWorker)
 
 
 @main.command(cls=EncloseNegative)
 @click.argument("number", type=int)
-@click.option("-c", "--copy", type=click.IntRange(1, 3),
-              help="Copies the answer. 1 for exact_ans, 2 for approx_ans, and 3 for a list of [exact_ans, "
-                   "approx_ans].")
-def pf(number, **kwargs):
+@click.option(
+    "-c",
+    "--copy",
+    type=click.IntRange(1, 3),
+    help="Copies the answer. 1 for exact_ans, 2 for approx_ans, and 3 for a list of [exact_ans, "
+    "approx_ans].",
+)
+def pf(number: int, **kwargs: dict) -> None:
     """Retreives the prime factors of an positive integer.
 
     Note: exact_ans stores factors as dict: '{2: 2, 3: 1, 31: 1}'
     while approx_ans stores factors as string: '(2**2)*(3**1)*(31**1)'
 
-    Usage: pf NUMBER [FLAGS]
-
     \b
     Example(s):
     >>> caspy pf 372
     """
     from .qt_assets.tabs.pf import PfWorker
+
     to_send = ["calc_pf", [number], kwargs["copy"]]
     send_to_thread(to_send, PfWorker)
 
 
 @main.command(cls=EncloseNegative)
-@click.argument("website_index", type=int, required=False)
-@click.option("--list", "-l", is_flag=True)
-def web(website_index, list):
-    """Choose a number from a list of usable maths websites and open it in default web browser.
-
-    type '-l' for a list of websites and enter a number. The website will be opened in the default browser.
-
-    Usage: web {NUMBER | LIST}
+@click.argument("website_index", type=int, required=False, metavar="{NUMBER | LIST}")
+@click.option("--list-websites", "-l", is_flag=True, default=False)
+def web(website_index: int, list_websites: bool, **kwargs: dict) -> None:
+    """Choose a number from a list of usable maths websites and open it in default web browser or
+    type '-l' for a list of websites and then enter a number. The website will be opened in the default browser.
 
     \b
     Example(s):
     >>> caspy web 4
     >>> caspy web -l
     """
-
     import json
     import pkg_resources
-    with open(pkg_resources.resource_filename('caspy3', "data/websites.json"), "r", encoding="utf8") as json_f:
+
+    with open(
+        pkg_resources.resource_filename("caspy3", "data/websites.json"),
+        "r",
+        encoding="utf8",
+    ) as json_f:
         json_data = json_f.read()
         web_list = json.loads(json_data)
 
@@ -645,18 +849,19 @@ def web(website_index, list):
             print(f"Index of website must be between 1 and {len(web_list)}")
             return
 
-    if list:
+    if list_websites:
         for web, i in zip(web_list, range(len(web_list))):
             print(f"{i + 1}. {next(iter(web))}")
 
     else:
         import webbrowser
+
         url = next(iter(web_list[website_index - 1].values()))
         webbrowser.open(url)
 
 
-def send_to_thread(input_list, worker):
-    def excepthook(exc_type, exc_value, exc_tb):
+def send_to_thread(input_list: list, worker: ty.Type[BaseWorker]) -> None:
+    def excepthook(exc_type, exc_value, exc_tb) -> None:
         tb = "".join(traceback.format_exception(exc_type, exc_value, exc_tb))
         print("error catched!:")
         print("error message:\n", tb)
