@@ -1,24 +1,51 @@
-from PyQt5.QtCore import pyqtSlot, QEvent, Qt
-from PyQt5.QtWidgets import QApplication, QWidget
-from PyQt5.QtGui import QCursor
+#
+#    CASPy - A program that provides both a GUI and a CLI to SymPy.
+#    Copyright (C) 2020 Folke Ishii
+#
+#    This program is free software: you can redistribute it and/or modify
+#    it under the terms of the GNU General Public License as published by
+#    the Free Software Foundation, either version 3 of the License, or
+#    (at your option) any later version.
+#
+#    This program is distributed in the hope that it will be useful,
+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#    GNU General Public License for more details.
+#
+#    You should have received a copy of the GNU General Public License
+#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
+#
+
+from PyQt5.QtCore import pyqtSlot, Qt
+from PyQt5.QtWidgets import QShortcut, QTextBrowser, QWidget
+from PyQt5.QtGui import QCursor, QKeySequence
 from PyQt5.uic import loadUi
 
 from sympy import *
 from sympy.parsing.sympy_parser import parse_expr
 
 import traceback
+import typing as ty
 
 from .worker import BaseWorker
 
 
 class DerivativeWorker(BaseWorker):
-    def __init__(self, command, params, copy=None):
+    def __init__(self, command: str, params: list, copy: int = None) -> None:
         super().__init__(command, params, copy)
 
     @BaseWorker.catch_error
     @pyqtSlot()
-    def prev_deriv(self, input_expression, input_variable, input_order, input_point, output_type, use_unicode,
-                   line_wrap):
+    def prev_deriv(
+        self,
+        input_expression: str,
+        input_variable: str,
+        input_order: int,
+        input_point: str,
+        output_type: int,
+        use_unicode: bool,
+        line_wrap: bool,
+    ) -> ty.Dict[str, ty.List[str]]:
         init_printing(use_unicode=use_unicode, wrap_line=line_wrap)
         self.approx_ans = 0
         self.exact_ans = ""
@@ -49,8 +76,18 @@ class DerivativeWorker(BaseWorker):
 
     @BaseWorker.catch_error
     @pyqtSlot()
-    def calc_deriv(self, input_expression, input_variable, input_order, input_point, output_type, use_unicode,
-                   line_wrap, use_scientific, accuracy):
+    def calc_deriv(
+        self,
+        input_expression: str,
+        input_variable: str,
+        input_order: int,
+        input_point: str,
+        output_type: int,
+        use_unicode: bool,
+        line_wrap: bool,
+        use_scientific: ty.Union[int, None],
+        accuracy: int,
+    ) -> ty.Dict[str, ty.List[str]]:
         init_printing(use_unicode=use_unicode, wrap_line=line_wrap)
 
         self.approx_ans = 0
@@ -67,17 +104,23 @@ class DerivativeWorker(BaseWorker):
             return {"error": ["Enter a variable"]}
 
         try:
-            self.exact_ans = diff(parse_expr(input_expression), parse_expr(input_variable), input_order)
+            self.exact_ans = diff(
+                parse_expr(input_expression), parse_expr(input_variable), input_order
+            )
         except Exception:
             return {"error": [f"Error: \n{traceback.format_exc()}"]}
         self.latex_answer = str(latex(self.exact_ans))
 
         if input_point:
-            calc_deriv_point = str(self.exact_ans).replace(input_variable, f"({input_point})")
+            calc_deriv_point = str(self.exact_ans).replace(
+                input_variable, f"({input_point})"
+            )
 
             if use_scientific:
                 try:
-                    self.approx_ans = self.to_scientific_notation(str(N(calc_deriv_point, accuracy)), use_scientific)
+                    self.approx_ans = self.to_scientific_notation(
+                        str(N(calc_deriv_point, accuracy)), use_scientific
+                    )
                 except Exception:
                     return {"error": [f"Failed to parse {input_point}"]}
             else:
@@ -108,7 +151,7 @@ class DerivativeTab(QWidget):
 
     display_name = "Derivative"
 
-    def __init__(self, main_window):
+    def __init__(self, main_window: "CASpyGUI") -> None:
         """
         A QWidget is created to be added in as a tab in the main window.
 
@@ -123,39 +166,34 @@ class DerivativeTab(QWidget):
         super().__init__()
         self.main_window = main_window
         loadUi(self.main_window.get_resource_path("qt_assets/tabs/derivative.ui"), self)
+        self.eout: QTextBrowser = self.DerivOut
+        self.aout: QTextBrowser = self.DerivApprox
 
-        self.install_event_filters()
+        # Shortcuts
+        cshortcut = QShortcut(QKeySequence("Ctrl+Return"), self)
+        cshortcut.activated.connect(self.calc_deriv)
+        pshortcut = QShortcut(QKeySequence("Ctrl+Shift+Return"), self)
+        pshortcut.activated.connect(self.prev_deriv)
         self.init_bindings()
 
-    def install_event_filters(self):
-        self.DerivExp.installEventFilter(self)
-
-    def eventFilter(self, obj, event):
-        """
-        Add modifiers and if shift + enter or shift + return is pressed, run calc_deriv()
-        """
-        QModifiers = QApplication.keyboardModifiers()
-        modifiers = []
-        if (QModifiers & Qt.ShiftModifier) == Qt.ShiftModifier:
-            modifiers.append('shift')
-
-        if event.type() == QEvent.KeyPress:
-            if event.key() in (Qt.Key_Return, Qt.Key_Enter):
-                if modifiers:
-                    if modifiers[0] == "shift":
-                        self.calc_deriv()
-                        return True
-
-        return super(DerivativeTab, self).eventFilter(obj, event)
-
-    def init_bindings(self):
+    def init_bindings(self) -> None:
         self.DerivPrev.clicked.connect(self.prev_deriv)
         self.DerivCalc.clicked.connect(self.calc_deriv)
+        self.eout.mousePressEvent = lambda _: self.eout.selectAll()
+        self.aout.mousePressEvent = lambda _: self.aout.selectAll()
+        self.eout.focusOutEvent = lambda _: self.deselect(self.eout)
+        self.aout.focusOutEvent = lambda _: self.deselect(self.aout)
 
-    def stop_thread(self):
+    @staticmethod
+    def deselect(textbrowser: "QTextBrowser") -> None:
+        cursor = textbrowser.textCursor()
+        cursor.clearSelection()
+        textbrowser.setTextCursor(cursor)
+
+    def stop_thread(self) -> None:
         pass
 
-    def update_ui(self, input_dict):
+    def update_ui(self, input_dict: ty.Dict[str, ty.List[str]]) -> None:
         self.DerivOut.viewport().setProperty("cursor", QCursor(Qt.ArrowCursor))
         self.DerivApprox.viewport().setProperty("cursor", QCursor(Qt.ArrowCursor))
 
@@ -171,48 +209,45 @@ class DerivativeTab(QWidget):
             self.DerivOut.setText(self.main_window.exact_ans)
             self.DerivApprox.setText(str(self.main_window.approx_ans))
 
-    def prev_deriv(self):
+    def prev_deriv(self) -> None:
         self.DerivOut.viewport().setProperty("cursor", QCursor(Qt.WaitCursor))
         self.DerivApprox.viewport().setProperty("cursor", QCursor(Qt.WaitCursor))
 
-        worker = DerivativeWorker("prev_deriv", [
-            self.DerivExp.toPlainText(),
-            self.DerivVar.text(),
-            self.DerivOrder.value(),
-            self.DerivPoint.text(),
-            self.main_window.output_type,
-            self.main_window.use_unicode,
-            self.main_window.line_wrap
-        ])
+        worker = DerivativeWorker(
+            "prev_deriv",
+            [
+                self.DerivExp.toPlainText(),
+                self.DerivVar.text(),
+                self.DerivOrder.value(),
+                self.DerivPoint.text(),
+                self.main_window.output_type,
+                self.main_window.use_unicode,
+                self.main_window.line_wrap,
+            ],
+        )
         worker.signals.output.connect(self.update_ui)
         worker.signals.finished.connect(self.stop_thread)
 
         self.main_window.threadpool.start(worker)
 
-    def calc_deriv_new(self):
+    def calc_deriv(self) -> None:
         self.DerivOut.viewport().setProperty("cursor", QCursor(Qt.WaitCursor))
         self.DerivApprox.viewport().setProperty("cursor", QCursor(Qt.WaitCursor))
 
-        worker = DerivativeWorker()
-
-        worker.signals.output.connect(self.update_ui)
-        worker.signals.finished.connect(self.stop_thread)
-
-    def calc_deriv(self):
-        self.DerivOut.viewport().setProperty("cursor", QCursor(Qt.WaitCursor))
-        self.DerivApprox.viewport().setProperty("cursor", QCursor(Qt.WaitCursor))
-
-        worker = DerivativeWorker("calc_deriv", [
-            self.DerivExp.toPlainText(),
-            self.DerivVar.text(),
-            self.DerivOrder.value(),
-            self.DerivPoint.text(),
-            self.main_window.output_type,
-            self.main_window.use_unicode,
-            self.main_window.line_wrap,
-            self.main_window.use_scientific,
-            self.main_window.accuracy
-        ])
+        worker = DerivativeWorker(
+            "calc_deriv",
+            [
+                self.DerivExp.toPlainText(),
+                self.DerivVar.text(),
+                self.DerivOrder.value(),
+                self.DerivPoint.text(),
+                self.main_window.output_type,
+                self.main_window.use_unicode,
+                self.main_window.line_wrap,
+                self.main_window.use_scientific,
+                self.main_window.accuracy,
+            ],
+        )
         worker.signals.output.connect(self.update_ui)
         worker.signals.finished.connect(self.stop_thread)
 
