@@ -1,8 +1,28 @@
-from PyQt5.QtCore import pyqtSlot, QEvent, Qt
-from PyQt5.QtWidgets import QApplication, QWidget
-from PyQt5.QtGui import QCursor
+#
+#    CASPy - A program that provides both a GUI and a CLI to SymPy.
+#    Copyright (C) 2020 Folke Ishii
+#
+#    This program is free software: you can redistribute it and/or modify
+#    it under the terms of the GNU General Public License as published by
+#    the Free Software Foundation, either version 3 of the License, or
+#    (at your option) any later version.
+#
+#    This program is distributed in the hope that it will be useful,
+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#    GNU General Public License for more details.
+#
+#    You should have received a copy of the GNU General Public License
+#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
+#
+
+from PyQt5.QtCore import pyqtSlot, Qt
+from PyQt5.QtWidgets import QShortcut, QWidget
+from PyQt5.QtGui import QCursor, QKeySequence
 from PyQt5.uic import loadUi
 
+import typing as ty
+
 from sympy import *
 from sympy.parsing.sympy_parser import parse_expr
 
@@ -12,12 +32,14 @@ from .worker import BaseWorker
 
 
 class ExpandWorker(BaseWorker):
-    def __init__(self, command, params, copy=None):
+    def __init__(self, command: str, params: list, copy: int = None) -> None:
         super().__init__(command, params, copy)
 
     @BaseWorker.catch_error
     @pyqtSlot()
-    def prev_expand_exp(self, expression, output_type, use_unicode, line_wrap):
+    def prev_expand_exp(
+        self, expression: str, output_type: int, use_unicode: bool, line_wrap: bool
+    ) -> ty.Dict[str, ty.List[str]]:
         init_printing(use_unicode=use_unicode, wrap_line=line_wrap)
         self.approx_ans = 0
         self.exact_ans = ""
@@ -45,7 +67,9 @@ class ExpandWorker(BaseWorker):
 
     @BaseWorker.catch_error
     @pyqtSlot()
-    def expand_exp(self, expression, output_type, use_unicode, line_wrap):
+    def expand_exp(
+        self, expression: str, output_type: int, use_unicode: bool, line_wrap: bool
+    ) -> ty.Dict[str, ty.List[str]]:
         init_printing(use_unicode=use_unicode, wrap_line=line_wrap)
         self.approx_ans = 0
         self.exact_ans = ""
@@ -73,40 +97,37 @@ class ExpandWorker(BaseWorker):
 class ExpandTab(QWidget):
     display_name = "Expand"
 
-    def __init__(self, main_window):
+    def __init__(self, main_window: "CASpyGUI") -> None:
         super().__init__()
         self.main_window = main_window
         loadUi(self.main_window.get_resource_path("qt_assets/tabs/expand.ui"), self)
+        self.eout = self.ExpOut
+        self.aout = None
+
+        # Shortcuts
+        cshortcut = QShortcut(QKeySequence("Ctrl+Return"), self)
+        cshortcut.activated.connect(self.expand_exp)
+        pshortcut = QShortcut(QKeySequence("Ctrl+Shift+Return"), self)
+        pshortcut.activated.connect(self.prev_expand_exp)
 
-        self.install_event_filters()
         self.init_bindings()
 
-    def install_event_filters(self):
-        self.ExpExp.installEventFilter(self)
-
-    def eventFilter(self, obj, event):
-        QModifiers = QApplication.keyboardModifiers()
-        modifiers = []
-        if (QModifiers & Qt.ShiftModifier) == Qt.ShiftModifier:
-            modifiers.append('shift')
-
-        if event.type() == QEvent.KeyPress:
-            if event.key() in (Qt.Key_Return, Qt.Key_Enter):
-                if modifiers:
-                    if modifiers[0] == "shift":
-                        self.expand_exp()
-                        return True
-
-        return super(ExpandTab, self).eventFilter(obj, event)
-
-    def init_bindings(self):
+    def init_bindings(self) -> None:
         self.ExpPrev.clicked.connect(self.prev_expand_exp)
         self.ExpCalc.clicked.connect(self.expand_exp)
+        self.eout.mousePressEvent = lambda _: self.eout.selectAll()
+        self.eout.focusOutEvent = lambda _: self.deselect(self.eout)
 
-    def stop_thread(self):
+    @staticmethod
+    def deselect(textbrowser: "QTextBrowser") -> None:
+        cursor = textbrowser.textCursor()
+        cursor.clearSelection()
+        textbrowser.setTextCursor(cursor)
+
+    def stop_thread(self) -> None:
         pass
 
-    def update_ui(self, input_dict):
+    def update_ui(self, input_dict: ty.Dict[str, ty.List[str]]) -> None:
         self.ExpOut.viewport().setProperty("cursor", QCursor(Qt.ArrowCursor))
 
         first_key = list(input_dict.keys())[0]
@@ -120,29 +141,35 @@ class ExpandTab(QWidget):
 
             self.ExpOut.setText(self.main_window.exact_ans)
 
-    def prev_expand_exp(self):
+    def prev_expand_exp(self) -> None:
         self.ExpOut.viewport().setProperty("cursor", QCursor(Qt.WaitCursor))
 
-        worker = ExpandWorker("prev_expand_exp", [
-            self.ExpExp.toPlainText(),
-            self.main_window.output_type,
-            self.main_window.use_unicode,
-            self.main_window.line_wrap
-        ])
+        worker = ExpandWorker(
+            "prev_expand_exp",
+            [
+                self.ExpExp.toPlainText(),
+                self.main_window.output_type,
+                self.main_window.use_unicode,
+                self.main_window.line_wrap,
+            ],
+        )
         worker.signals.output.connect(self.update_ui)
         worker.signals.finished.connect(self.stop_thread)
 
         self.main_window.threadpool.start(worker)
 
-    def expand_exp(self):
+    def expand_exp(self) -> None:
         self.ExpOut.viewport().setProperty("cursor", QCursor(Qt.WaitCursor))
 
-        worker = ExpandWorker("expand_exp", [
-            self.ExpExp.toPlainText(),
-            self.main_window.output_type,
-            self.main_window.use_unicode,
-            self.main_window.line_wrap
-        ])
+        worker = ExpandWorker(
+            "expand_exp",
+            [
+                self.ExpExp.toPlainText(),
+                self.main_window.output_type,
+                self.main_window.use_unicode,
+                self.main_window.line_wrap,
+            ],
+        )
         worker.signals.output.connect(self.update_ui)
         worker.signals.finished.connect(self.stop_thread)
 
