@@ -1,8 +1,28 @@
-from PyQt5.QtCore import pyqtSlot, QEvent, Qt
-from PyQt5.QtWidgets import QApplication, QWidget
-from PyQt5.QtGui import QCursor
+#
+#    CASPy - A program that provides both a GUI and a CLI to SymPy.
+#    Copyright (C) 2020 Folke Ishii
+#
+#    This program is free software: you can redistribute it and/or modify
+#    it under the terms of the GNU General Public License as published by
+#    the Free Software Foundation, either version 3 of the License, or
+#    (at your option) any later version.
+#
+#    This program is distributed in the hope that it will be useful,
+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#    GNU General Public License for more details.
+#
+#    You should have received a copy of the GNU General Public License
+#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
+#
+
+from PyQt5.QtCore import pyqtSlot, Qt
+from PyQt5.QtWidgets import QShortcut, QWidget
+from PyQt5.QtGui import QCursor, QKeySequence
 from PyQt5.uic import loadUi
 
+import typing as ty
+
 from sympy import *
 from sympy.parsing.sympy_parser import parse_expr
 
@@ -12,13 +32,21 @@ from .worker import BaseWorker
 
 
 class LimitWorker(BaseWorker):
-    def __init__(self, command, params, copy=None):
+    def __init__(self, command: str, params: list, copy: int = None) -> None:
         super().__init__(command, params, copy)
 
     @BaseWorker.catch_error
     @pyqtSlot()
-    def prev_limit(self, input_expression, input_variable, input_approach, input_side, output_type,
-                   use_unicode, line_wrap):
+    def prev_limit(
+        self,
+        input_expression: str,
+        input_variable: str,
+        input_approach: str,
+        input_side: str,
+        output_type: int,
+        use_unicode: bool,
+        line_wrap: bool,
+    ) -> ty.Dict[str, ty.List[str]]:
         init_printing(use_unicode=use_unicode, wrap_line=line_wrap)
         self.approx_ans = 0
         self.exact_ans = ""
@@ -32,7 +60,12 @@ class LimitWorker(BaseWorker):
             return {"error": ["Enter a variable"]}
 
         try:
-            self.exact_ans = Limit(parse_expr(input_expression), parse_expr(input_variable), input_approach, input_side)
+            self.exact_ans = Limit(
+                parse_expr(input_expression),
+                parse_expr(input_variable),
+                input_approach,
+                input_side,
+            )
         except Exception:
             return {"error": [f"Error: \n{traceback.format_exc()}"]}
         self.latex_answer = str(latex(self.exact_ans))
@@ -48,8 +81,18 @@ class LimitWorker(BaseWorker):
 
     @BaseWorker.catch_error
     @pyqtSlot()
-    def calc_limit(self, input_expression, input_variable, input_approach, input_side, output_type,
-                   use_unicode, line_wrap, use_scientific, accuracy):
+    def calc_limit(
+        self,
+        input_expression: str,
+        input_variable: str,
+        input_approach: str,
+        input_side: str,
+        output_type: int,
+        use_unicode: bool,
+        line_wrap: bool,
+        use_scientific: ty.Union[int, None],
+        accuracy: int,
+    ) -> ty.Dict[str, ty.List[str]]:
         init_printing(use_unicode=use_unicode, wrap_line=line_wrap)
         self.approx_ans = 0
         self.exact_ans = ""
@@ -67,13 +110,20 @@ class LimitWorker(BaseWorker):
             return {"error": ["Enter a variable"]}
 
         try:
-            self.exact_ans = limit(parse_expr(input_expression), parse_expr(input_variable), input_approach, input_side)
+            self.exact_ans = limit(
+                parse_expr(input_expression),
+                parse_expr(input_variable),
+                input_approach,
+                input_side,
+            )
         except Exception:
             return {"error": [f"Error: \n{traceback.format_exc()}"]}
         self.latex_answer = str(latex(self.exact_ans))
 
         if use_scientific:
-            self.approx_ans = self.to_scientific_notation(str(N(self.exact_ans, accuracy)), use_scientific)
+            self.approx_ans = self.to_scientific_notation(
+                str(N(self.exact_ans, accuracy)), use_scientific
+            )
         else:
             self.approx_ans = str(N(self.exact_ans, accuracy))
 
@@ -91,40 +141,39 @@ class LimitTab(QWidget):
 
     display_name = "Limit"
 
-    def __init__(self, main_window):
+    def __init__(self, main_window: "CASpyGUI") -> None:
         super().__init__()
         self.main_window = main_window
         loadUi(self.main_window.get_resource_path("qt_assets/tabs/limit.ui"), self)
+        self.eout = self.LimOut
+        self.aout = self.LimApprox
+
+        # Shortcuts
+        cshortcut = QShortcut(QKeySequence("Ctrl+Return"), self)
+        cshortcut.activated.connect(self.calc_limit)
+        pshortcut = QShortcut(QKeySequence("Ctrl+Shift+Return"), self)
+        pshortcut.activated.connect(self.prev_limit)
 
-        self.install_event_filters()
         self.init_bindings()
 
-    def install_event_filters(self):
-        self.LimExp.installEventFilter(self)
-
-    def eventFilter(self, obj, event):
-        QModifiers = QApplication.keyboardModifiers()
-        modifiers = []
-        if (QModifiers & Qt.ShiftModifier) == Qt.ShiftModifier:
-            modifiers.append('shift')
-
-        if event.type() == QEvent.KeyPress:
-            if event.key() in (Qt.Key_Return, Qt.Key_Enter):
-                if modifiers:
-                    if modifiers[0] == "shift":
-                        self.calc_limit()
-                        return True
-
-        return super(LimitTab, self).eventFilter(obj, event)
-
-    def init_bindings(self):
+    def init_bindings(self) -> None:
         self.LimPrev.clicked.connect(self.prev_limit)
         self.LimCalc.clicked.connect(self.calc_limit)
+        self.eout.mousePressEvent = lambda _: self.eout.selectAll()
+        self.aout.mousePressEvent = lambda _: self.aout.selectAll()
+        self.eout.focusOutEvent = lambda _: self.deselect(self.eout)
+        self.aout.focusOutEvent = lambda _: self.deselect(self.aout)
 
-    def stop_thread(self):
+    @staticmethod
+    def deselect(textbrowser: "QTextBrowser") -> None:
+        cursor = textbrowser.textCursor()
+        cursor.clearSelection()
+        textbrowser.setTextCursor(cursor)
+
+    def stop_thread(self) -> None:
         pass
 
-    def update_ui(self, input_dict):
+    def update_ui(self, input_dict: ty.Dict[str, ty.List[str]]) -> None:
         self.LimOut.viewport().setProperty("cursor", QCursor(Qt.ArrowCursor))
         self.LimApprox.viewport().setProperty("cursor", QCursor(Qt.ArrowCursor))
 
@@ -140,7 +189,7 @@ class LimitTab(QWidget):
             self.LimOut.setText(self.main_window.exact_ans)
             self.LimApprox.setText(str(self.main_window.approx_ans))
 
-    def prev_limit(self):
+    def prev_limit(self) -> None:
         self.LimOut.viewport().setProperty("cursor", QCursor(Qt.WaitCursor))
         self.LimApprox.viewport().setProperty("cursor", QCursor(Qt.WaitCursor))
 
@@ -151,21 +200,24 @@ class LimitTab(QWidget):
         else:
             limit_side = "+"
 
-        worker = LimitWorker("prev_limit", [
-            self.LimExp.toPlainText(),
-            self.LimVar.text(),
-            self.LimApproach.text(),
-            limit_side,
-            self.main_window.output_type,
-            self.main_window.use_unicode,
-            self.main_window.line_wrap
-        ])
+        worker = LimitWorker(
+            "prev_limit",
+            [
+                self.LimExp.toPlainText(),
+                self.LimVar.text(),
+                self.LimApproach.text(),
+                limit_side,
+                self.main_window.output_type,
+                self.main_window.use_unicode,
+                self.main_window.line_wrap,
+            ],
+        )
         worker.signals.output.connect(self.update_ui)
         worker.signals.finished.connect(self.stop_thread)
 
         self.main_window.threadpool.start(worker)
 
-    def calc_limit(self):
+    def calc_limit(self) -> None:
         self.LimOut.viewport().setProperty("cursor", QCursor(Qt.WaitCursor))
         self.LimApprox.viewport().setProperty("cursor", QCursor(Qt.WaitCursor))
 
@@ -176,17 +228,20 @@ class LimitTab(QWidget):
         else:
             limit_side = "+"
 
-        worker = LimitWorker("calc_limit", [
-            self.LimExp.toPlainText(),
-            self.LimVar.text(),
-            self.LimApproach.text(),
-            limit_side,
-            self.main_window.output_type,
-            self.main_window.use_unicode,
-            self.main_window.line_wrap,
-            self.main_window.use_scientific,
-            self.main_window.accuracy
-        ])
+        worker = LimitWorker(
+            "calc_limit",
+            [
+                self.LimExp.toPlainText(),
+                self.LimVar.text(),
+                self.LimApproach.text(),
+                limit_side,
+                self.main_window.output_type,
+                self.main_window.use_unicode,
+                self.main_window.line_wrap,
+                self.main_window.use_scientific,
+                self.main_window.accuracy,
+            ],
+        )
         worker.signals.output.connect(self.update_ui)
         worker.signals.finished.connect(self.stop_thread)
 
